@@ -55,7 +55,7 @@ class TimedBelief(object):
         sensor: Sensor,
         source: Union[BeliefSource, str, int],
         event_value: Optional[float] = None,
-        value: Optional[float] = None,
+        value: Optional[float] = None,  # deprecated
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
@@ -192,7 +192,7 @@ class TimedBeliefDBMixin(TimedBelief):
         sensor: DBSensor,
         source: DBBeliefSource,
         event_value: Optional[float] = None,
-        value: Optional[float] = None,
+        value: Optional[float] = None,  # deprecated
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
@@ -226,15 +226,30 @@ class TimedBeliefDBMixin(TimedBelief):
         cls,
         session: Session,
         beliefs_data_frame: "BeliefsDataFrame",
+        expunge_session: bool = False,
+        allow_overwrite: bool = False,
+        bulk_save_objects: bool = False,
         commit_transaction: bool = False,
     ):
         """Add a BeliefsDataFrame as timed beliefs to a database session.
 
-        :param session: the database session to use
-        :param beliefs_data_frame: the BeliefsDataFrame to be persisted
-        :param commit_transaction: if True, the session is committed
-                                   if False, you can still add other data to the session
-                                   and commit it all within an atomic transaction
+        If you are adding lots of beliefs, it's most efficient to use expunge_session=True and allow_overwrite=False
+
+        :param session:             the database session to use
+        :param beliefs_data_frame:  the BeliefsDataFrame to be persisted
+        :param expunge_session:     if True, all non-flushed instances are removed from the session before adding beliefs.
+                                    Expunging can resolve problems you might encounter with states of objects in your session.
+                                    When using this option, you might want to flush newly-created objects which are not beliefs
+                                    (e.g. a sensor or data source object).
+        :param allow_overwrite:     if True, new objects are merged
+                                    if False, objects are added to the session or bulk saved
+        :param bulk_save_objects:   if True, objects are bulk saved with session.bulk_save_objects(),
+                                    which is quite fast but has several caveats, see:
+                                    https://docs.sqlalchemy.org/orm/persistence_techniques.html#bulk-operations-caveats
+                                    if False, objects are added to the session with session.add_all()
+        :param commit_transaction:  if True, the session is committed
+                                    if False, you can still add other data to the session
+                                    and commit it all within an atomic transaction
         """
         # Belief timing is stored as the belief horizon rather than as the belief time
         belief_records = (
@@ -243,7 +258,16 @@ class TimedBeliefDBMixin(TimedBelief):
             .to_dict("records")
         )
         beliefs = [cls(sensor=beliefs_data_frame.sensor, **d) for d in belief_records]
-        session.add_all(beliefs)
+        if expunge_session:
+            session.expunge_all()
+        if not allow_overwrite:
+            if bulk_save_objects:
+                session.bulk_save_objects(beliefs)
+            else:
+                session.add_all(beliefs)
+        else:
+            for belief in beliefs:
+                session.merge(belief)
         if commit_transaction:
             session.commit()
 
@@ -265,37 +289,78 @@ class TimedBeliefDBMixin(TimedBelief):
         cls,
         session: Session,
         sensor: SensorDBMixin,
-        event_before: Optional[datetime] = None,
-        event_not_before: Optional[datetime] = None,
-        belief_before: Optional[datetime] = None,
-        belief_not_before: Optional[datetime] = None,
+        event_starts_after: Optional[datetime] = None,
+        event_ends_before: Optional[datetime] = None,
+        beliefs_after: Optional[datetime] = None,
+        beliefs_before: Optional[datetime] = None,
+        event_before: Optional[datetime] = None,  # deprecated
+        event_not_before: Optional[datetime] = None,  # deprecated
+        belief_before: Optional[datetime] = None,  # deprecated
+        belief_not_before: Optional[datetime] = None,  # deprecated
         source: Optional[Union[BeliefSource, List[BeliefSource]]] = None,
     ) -> "BeliefsDataFrame":
         """Search a database session for beliefs about sensor events.
+
+        The optional arguments represent optional filters.
         :param session: the database session to use
         :param sensor: sensor to which the beliefs pertain
-        :param event_before: only return beliefs about events that end before this datetime (inclusive)
-        :param event_not_before: only return beliefs about events that start after this datetime (inclusive)
-        :param belief_before: only return beliefs formed before this datetime (inclusive)
-        :param belief_not_before: only return beliefs formed after this datetime (inclusive)
+        :param event_starts_after: only return beliefs about events that start after this datetime (inclusive)
+        :param event_ends_before: only return beliefs about events that end before this datetime (inclusive)
+        :param beliefs_after: only return beliefs formed after this datetime (inclusive)
+        :param beliefs_before: only return beliefs formed before this datetime (inclusive)
         :param source: only return beliefs formed by the given source or list of sources
         :returns: a multi-index DataFrame with all relevant beliefs
-
-        TODO: rename params for clarity: event_finished_before, event_starts_not_before (or similar), same for beliefs
         """
 
+        # todo: deprecate the 'event_before' argument in favor of 'event_ends_before' (announced v1.4.1)
+        event_ends_before = tb_utils.replace_deprecated_argument(
+            "event_before",
+            event_before,
+            "event_ends_before",
+            event_ends_before,
+            required_argument=False,
+        )
+        # todo: deprecate the 'event_not_before' argument in favor of 'event_starts_after' (announced v1.4.1)
+        event_starts_after = tb_utils.replace_deprecated_argument(
+            "event_not_before",
+            event_not_before,
+            "event_starts_after",
+            event_starts_after,
+            required_argument=False,
+        )
+        # todo: deprecate the 'belief_before' argument in favor of 'beliefs_before' (announced v1.4.1)
+        beliefs_before = tb_utils.replace_deprecated_argument(
+            "belief_before",
+            belief_before,
+            "beliefs_before",
+            beliefs_before,
+            required_argument=False,
+        )
+        # todo: deprecate the 'belief_not_before' argument in favor of 'beliefs_after' (announced v1.4.1)
+        beliefs_after = tb_utils.replace_deprecated_argument(
+            "belief_not_before",
+            belief_not_before,
+            "beliefs_after",
+            beliefs_after,
+            required_argument=False,
+        )
+
         # Check for timezone-aware datetime input
-        if event_before is not None:
-            event_before = tb_utils.parse_datetime_like(event_before, "event_before")
-        if event_not_before is not None:
-            event_not_before = tb_utils.parse_datetime_like(
-                event_not_before, "event_not_before"
+        if event_starts_after is not None:
+            event_starts_after = tb_utils.parse_datetime_like(
+                event_starts_after, "event_not_before"
             )
-        if belief_before is not None:
-            belief_before = tb_utils.parse_datetime_like(belief_before, "belief_before")
-        if belief_not_before is not None:
-            belief_not_before = tb_utils.parse_datetime_like(
-                belief_not_before, "belief_not_before"
+        if event_ends_before is not None:
+            event_ends_before = tb_utils.parse_datetime_like(
+                event_ends_before, "event_before"
+            )
+        if beliefs_after is not None:
+            beliefs_after = tb_utils.parse_datetime_like(
+                beliefs_after, "belief_not_before"
+            )
+        if beliefs_before is not None:
+            beliefs_before = tb_utils.parse_datetime_like(
+                beliefs_before, "belief_before"
             )
 
         # Query sensor for relevant timing properties
@@ -324,25 +389,27 @@ class TimedBeliefDBMixin(TimedBelief):
         q = session.query(cls).filter(cls.sensor_id == sensor.id)
 
         # Apply event time filter
-        if event_before is not None:
-            q = q.filter(cls.event_start + event_resolution <= event_before)
-        if event_not_before is not None:
-            q = q.filter(cls.event_start >= event_not_before)
+        if event_starts_after is not None:
+            q = q.filter(cls.event_start >= event_starts_after)
+        if event_ends_before is not None:
+            q = q.filter(cls.event_start + event_resolution <= event_ends_before)
 
         # Apply rough belief time filter
-        if belief_before is not None:
+        if beliefs_after is not None:
             q = q.filter(
                 cls.event_start
-                <= belief_before + cls.belief_horizon + knowledge_horizon_max
+                >= beliefs_after + cls.belief_horizon + knowledge_horizon_min
             )
-        if belief_not_before is not None:
+        if beliefs_before is not None:
             q = q.filter(
                 cls.event_start
-                >= belief_not_before + cls.belief_horizon + knowledge_horizon_min
+                <= beliefs_before + cls.belief_horizon + knowledge_horizon_max
             )
 
         # Apply source filter
-        if source is not None:
+        if source is []:
+            return BeliefsDataFrame(sensor=sensor, beliefs=[])
+        elif source is not None:
             sources: list = [source] if not isinstance(source, list) else source
             source_cls = sources[0].__class__
             q = q.join(source_cls).filter(cls.source_id.in_([s.id for s in sources]))
@@ -351,10 +418,10 @@ class TimedBeliefDBMixin(TimedBelief):
         df = BeliefsDataFrame(sensor=sensor, beliefs=q.all())
 
         # Actually filter by belief time
-        if belief_before is not None:
-            df = df[df.index.get_level_values("belief_time") < belief_before]
-        if belief_not_before is not None:
-            df = df[df.index.get_level_values("belief_time") >= belief_not_before]
+        if beliefs_after is not None:
+            df = df[df.index.get_level_values("belief_time") >= beliefs_after]
+        if beliefs_before is not None:
+            df = df[df.index.get_level_values("belief_time") < beliefs_before]
 
         return df
 
@@ -386,7 +453,7 @@ class DBTimedBelief(Base, TimedBeliefDBMixin):
         sensor: DBSensor,
         source: DBBeliefSource,
         event_value: Optional[float] = None,
-        value: Optional[float] = None,
+        value: Optional[float] = None,  # deprecated
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
@@ -423,7 +490,7 @@ class BeliefsSeries(pd.Series):
     @property
     def _constructor(self):
         def f(*args, **kwargs):
-            """ Call __finalize__() after construction to inherit metadata. """
+            """Call __finalize__() after construction to inherit metadata."""
             return BeliefsSeries(*args, **kwargs).__finalize__(self, method="inherit")
 
         return f
@@ -431,7 +498,7 @@ class BeliefsSeries(pd.Series):
     @property
     def _constructor_expanddim(self):
         def f(*args, **kwargs):
-            """ Call __finalize__() after construction to inherit metadata. """
+            """Call __finalize__() after construction to inherit metadata."""
             # adapted from https://github.com/pandas-dev/pandas/issues/19850#issuecomment-367934440
             return BeliefsDataFrame(*args, **kwargs).__finalize__(
                 self, method="inherit"
@@ -487,7 +554,7 @@ class BeliefsDataFrame(pd.DataFrame):
     @property
     def _constructor(self):
         def f(*args, **kwargs):
-            """ Call __finalize__() after construction to inherit metadata. """
+            """Call __finalize__() after construction to inherit metadata."""
             return BeliefsDataFrame(*args, **kwargs).__finalize__(
                 self, method="inherit"
             )
@@ -497,7 +564,7 @@ class BeliefsDataFrame(pd.DataFrame):
     @property
     def _constructor_sliced(self):
         def f(*args, **kwargs):
-            """ Call __finalize__() after construction to inherit metadata. """
+            """Call __finalize__() after construction to inherit metadata."""
             # adapted from https://github.com/pandas-dev/pandas/issues/19850#issuecomment-367934440
             return BeliefsSeries(*args, **kwargs).__finalize__(self, method="inherit")
 
@@ -784,7 +851,10 @@ class BeliefsDataFrame(pd.DataFrame):
         if "belief_horizon" in self.index.names:
             return self.index.get_level_values("belief_horizon")
         else:
-            return (self.knowledge_times - self.belief_times).rename("belief_horizon")
+            return (
+                self.knowledge_times.tz_convert("UTC")
+                - self.belief_times.tz_convert("UTC")
+            ).rename("belief_horizon")
 
     @property
     def event_starts(self) -> pd.DatetimeIndex:
@@ -1228,7 +1298,8 @@ class BeliefsDataFrame(pd.DataFrame):
 
         >>> from datetime import datetime
         >>> from pytz import utc
-        >>> from timely_beliefs.examples import example_df as df
+        >>> from timely_beliefs.examples import get_example_df
+        >>> df = get_example_df()
         >>> # Select the accuracy of beliefs held about each event on June 2nd (midnight)
         >>> df.fixed_viewpoint_accuracy(belief_time=datetime(2013, 6, 2, tzinfo=utc))
         >>> # Or equivalently:
@@ -1352,6 +1423,7 @@ class BeliefsDataFrame(pd.DataFrame):
         reference_source: BeliefSource = None,
         intuitive_forecast_horizon: bool = True,
         interpolate: bool = True,
+        event_value_range: Tuple[Optional[float], Optional[float]] = (None, None),
     ) -> alt.LayerChart:
         """Visualize the BeliefsDataFrame in an interactive Altair chart.
 
@@ -1362,6 +1434,13 @@ class BeliefsDataFrame(pd.DataFrame):
         :param intuitive_forecast_horizon: If true, horizons are shown with respect to event start rather than knowledge time
         :param interpolate: If True, the time series chart shows a user-friendly interpolated line
                rather than more accurate stripes indicating average values
+        :param event_value_range: Optionally set explicit limits on the range of event values (for axis scaling).
+               For example:
+               (0, 3)  # lower limit is 0, upper limit is 3
+               (None, 3)  # lower limit is taken from the plotted data, upper limit is 3
+               (0, None)  # lower limit is 0, upper limit is taken from the plotted data
+               (bdf["event_value"].min(), bdf["event_value"].max())  # limits are taken from the event value range of bdf
+               (None, None)  # default, limits are taken from the plotted data
         :returns: Altair chart object with a vega-lite representation (for more information, see reference below).
 
         >>> chart = df.plot(df.lineage.sources[0])
@@ -1380,6 +1459,7 @@ class BeliefsDataFrame(pd.DataFrame):
             reference_source=reference_source,
             intuitive_forecast_horizon=intuitive_forecast_horizon,
             interpolate=interpolate,
+            event_value_range=event_value_range,
         )
 
     @staticmethod
@@ -1501,7 +1581,7 @@ class BeliefsDataFrame(pd.DataFrame):
 
 
 def set_columns_and_indices_for_empty_frame(df, columns, indices, default_types):
-    """ Set appropriate columns and indices for the empty BeliefsDataFrame. """
+    """Set appropriate columns and indices for the empty BeliefsDataFrame."""
     if "belief_horizon" in df and "belief_time" not in df:
         indices = [
             "belief_horizon" if index == "belief_time" else index for index in indices
@@ -1525,7 +1605,7 @@ def set_columns_and_indices_for_empty_frame(df, columns, indices, default_types)
 
 
 def assign_sensor_and_event_resolution(df, sensor, event_resolution):
-    """ Set the Sensor metadata (including timing properties of the sensor). """
+    """Set the Sensor metadata (including timing properties of the sensor)."""
     df.sensor = sensor
     df.event_resolution = (
         event_resolution
